import { curry } from './curry';
const NOTHING = Symbol('nothing');
/**
 * Perform an operation with the previous value returned
 * @param withFn :: (a -> *)
 * @param fn: (...* -> a)
 * @returns (...* -> a)
 */
export const withPrevious = curry(function withPrevious(withFn, fn) {
    let previous = { [NOTHING]: true };
    return (...args) => {
        if (!isNothing(previous)) {
            withFn(previous.value);
        }
        const value = fn(...args);
        previous = { [NOTHING]: false, value };
        return value;
    };
});
function isNothing(maybe) {
    return maybe[NOTHING];
}
//# sourceMappingURL=withPrevious.js.map